@@ -22,8 +22,6 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: I18n.cc 4199 2006-02-16 06:53:05Z mathias $
-
 /* Note:
  * A good reference for the older non-gettext style I18n
  * functions is the "Locale tutorial"
@@ -38,6 +36,7 @@
 #endif // _GNU_SOURCE
 
 #include "I18n.hh"
+#include "FileUtil.hh"
 
 #include <X11/Xlocale.h>
 
@@ -66,23 +65,24 @@ using std::string;
 namespace FbTk {
 
 void NLSInit(const char *catalog) {
+    FbStringUtil::init();
     I18n *i18n = I18n::instance();
     i18n->openCatalog(catalog);
 }
 
 
-I18n::I18n():m_multibyte(false), m_catalog_fd((nl_catd)(-1)) {
-#ifdef 	HAVE_SETLOCALE
+I18n::I18n():m_multibyte(false), m_utf8_translate(false), m_catalog_fd((nl_catd)(-1)) {
+#if defined(HAVE_SETLOCALE) && defined(NLS)
     //make sure we don't get 0 to m_locale string
     char *temp = setlocale(LC_MESSAGES, "");
-    m_locale = ( temp ?  temp : ""); 
+    m_locale = ( temp ?  temp : "");
     if (m_locale.empty()) {
         cerr<<"Warning: Failed to set locale, reverting to \"C\""<<endl;
-#endif // HAVE_SETLOCALE
+#endif // defined(HAVE_SETLOCALE) && defined(NLS)
 
         m_locale = "C";
 
-#ifdef	HAVE_SETLOCALE
+#if defined(HAVE_SETLOCALE) && defined(NLS)
 
     } else {		
 
@@ -90,23 +90,19 @@ I18n::I18n():m_multibyte(false), m_catalog_fd((nl_catd)(-1)) {
         // MB_CUR_MAX returns the size of a char in the current locale
         if (MB_CUR_MAX > 1)
             m_multibyte = true;
-		
+
         // truncate any encoding off the end of the locale
-				
+
         // remove everything after @
         string::size_type index = m_locale.find('@');
         if (index != string::npos)
             m_locale.erase(index); //erase all characters starting at index 				
-        // remove everything after .		
-        index = m_locale.find('.');
-        if (index != string::npos) 
-            m_locale.erase(index); //erase all characters starting at index 
         // remove everything before =		
         index = m_locale.find('=');
         if (index != string::npos) 
             m_locale.erase(0,index+1); //erase all characters starting up to index 
     }
-#endif // HAVE_SETLOCALE
+#endif // defined(HAVE_SETLOCALE) && defined(NLS)
 }
 
 
@@ -125,13 +121,33 @@ I18n *I18n::instance() {
 
 void I18n::openCatalog(const char *catalog) {
 #if defined(NLS) && defined(HAVE_CATOPEN)
-	
+
     string catalog_filename = LOCALEPATH;
     catalog_filename += '/';
     catalog_filename += m_locale;
     catalog_filename += '/';
     catalog_filename += catalog;
 
+    if (!FileUtil::isRegularFile(catalog_filename.c_str()) && m_locale != "C" && FbStringUtil::haveUTF8()) {
+        // try the UTF-8 catalog, this also picks up situations where
+        // the codeset somehow isn't specified
+
+        // remove everything after @
+        string::size_type index = m_locale.find('.');
+        // erase all characters starting at index
+        if (index != string::npos)
+            m_locale.erase(index); 
+
+        m_locale.append(".UTF-8");
+        m_utf8_translate = true;
+
+        catalog_filename = LOCALEPATH;
+        catalog_filename += '/';
+        catalog_filename += m_locale;
+        catalog_filename += '/';
+        catalog_filename += catalog;
+    }
+
 #ifdef MCLoadBySet
     m_catalog_fd = catopen(catalog_filename.c_str(), MCLoadBySet);
 #else // !MCLoadBySet
@@ -150,15 +166,33 @@ void I18n::openCatalog(const char *catalog) {
 }
 
 
-const char *I18n::getMessage(int set_number, int message_number, 
-                             const char *default_message) const {
+// Translate_FB means it'll become an FbString that goes to X for Fonts, 
+// No translate means it stays in the local encoding, for printing to the
+// console.
+FbString I18n::getMessage(int set_number, int message_number, 
+                             const char *default_message, bool translate_fb) const {
 
 #if defined(NLS) && defined(HAVE_CATGETS)
-    if (m_catalog_fd != (nl_catd)-1)
-        return (const char *) catgets(m_catalog_fd, set_number, message_number, default_message);
+    if (m_catalog_fd != (nl_catd)-1) {
+        const char *ret = catgets(m_catalog_fd, set_number, message_number, default_message);
+        // can't translate, leave it in raw ascii (utf-8 compatible)
+        if (ret == default_message || ret == NULL)
+            return default_message;
+
+        if (!m_utf8_translate && translate_fb) 
+            // Local input, UTF-8 output
+            return FbStringUtil::LocaleStrToFb(ret);
+        else if (m_utf8_translate && !translate_fb)
+            // UTF-8 input, local output
+            return FbStringUtil::FbStrToLocale(ret);
+        else 
+            // UTF-8 input, UTF-8 output OR
+            // local input, local output
+            return ret;
+    }
     else
 #endif // NLS && HAVE_CATGETS
         return default_message;
 }
 
-}; // end namespace FbTk
+} // end namespace FbTk
