@@ -188,7 +188,7 @@ ImageControl::ImageControl(int screen_num,
     cache_max = cmax;
 
     if (cache_timeout && s_timed_cache) {
-        m_timer.setTimeout(cache_timeout);
+        m_timer.setTimeout(cache_timeout * FbTk::FbTime::IN_MILLISECONDS);
         RefCount<Command<void> > clean_cache(new SimpleCommand<ImageControl>(*this, &ImageControl::cleanCache));
         m_timer.setCommand(clean_cache);
         m_timer.start();
@@ -495,7 +495,7 @@ void ImageControl::createColorTable() {
         }
 
         if (m_colors_per_channel < 2 || num_colors > static_cast<unsigned>(1 << m_screen_depth)) {
-            fprintf(stderr, "ImageControl::ImageControl: invalid colormap size %ld "
+            fprintf(stderr, "ImageControl::ImageControl: invalid colormap size %zd "
                     "(%d/%d/%d) - reducing",
                     num_colors, m_colors_per_channel, m_colors_per_channel,
                     m_colors_per_channel);
@@ -543,7 +543,7 @@ void ImageControl::createColorTable() {
             }
 
             if (m_colors_per_channel < 2 || num_colors > static_cast<unsigned>(1 << m_screen_depth)) {
-                fprintf(stderr,"FbTk::ImageControl: invalid colormap size %ld "
+                fprintf(stderr,"FbTk::ImageControl: invalid colormap size %zd "
                         "(%d/%d/%d) - reducing",
                         num_colors, m_colors_per_channel, m_colors_per_channel,
                         m_colors_per_channel);
