@@ -1,5 +1,5 @@
 // Resource.cc
-// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,11 +19,10 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Resource.cc 3864 2005-01-24 18:02:34Z mathias $
-
 #include "XrmDatabaseHelper.hh"
 #include "Resource.hh"
 #include "I18n.hh"
+#include "StringUtil.hh"
 
 #include <iostream>
 #ifdef HAVE_CASSERT
@@ -32,7 +31,9 @@
   #include <assert.h>
 #endif
 
-using namespace std;
+using std::cerr;
+using std::endl;
+using std::string;
 
 namespace FbTk {
 
@@ -41,7 +42,11 @@ ResourceManager::ResourceManager(const char *filename, bool lock_db) :
  m_database(0),
  m_filename(filename ? filename : "")
 {
-    ensureXrmIsInitialize();
+    static bool xrm_initialized = false;
+    if (!xrm_initialized) {
+        XrmInitialize();
+        xrm_initialized = true;
+    }
 
     if (lock_db)
         lock();
@@ -52,14 +57,13 @@ ResourceManager::~ResourceManager() {
         delete m_database;
 }
 
-bool ResourceManager::m_init = false;
 
 /**
-  reloads all resources from resourcefile 
+  reloads all resources from resourcefile
   @return true on success else false
 */
 bool ResourceManager::load(const char *filename) {
-    m_filename = filename;
+    m_filename = StringUtil::expandFilename(filename).c_str();
 
     // force reload (lock will ensure it exists)
     if (m_database) {
@@ -72,23 +76,23 @@ bool ResourceManager::load(const char *filename) {
         unlock();
         return false;
     }
-        
+
     XrmValue value;
     char *value_type;
-	
+
     //get list and go throu all the resources and load them
     ResourceList::iterator i = m_resourcelist.begin();
-    ResourceList::iterator i_end = m_resourcelist.end();	
+    ResourceList::iterator i_end = m_resourcelist.end();
     for (; i != i_end; ++i) {
-	
+
         Resource_base *resource = *i;
         if (XrmGetResource(**m_database, resource->name().c_str(),
-                           resource->altName().c_str(), &value_type, &value))			
+                           resource->altName().c_str(), &value_type, &value))
             resource->setFromString(value.addr);
         else {
             _FB_USES_NLS;
-            cerr<<_FBTKTEXT(Error, FailedRead, "Failed to read", "Couldn't load a resource (following)")<<": "<<resource->name()<<endl;
-            cerr<<_FBTKTEXT(Error, UsingDefault, "Setting default value", "Falling back to default value for resource")<<endl;
+            cerr<<_FBTK_CONSOLETEXT(Error, FailedRead, "Failed to read", "Couldn't load a resource (following)")<<": "<<resource->name()<<endl;
+            cerr<<_FBTK_CONSOLETEXT(Error, UsingDefault, "Setting default value", "Falling back to default value for resource")<<endl;
             resource->setDefaultValue();
         }
     }
@@ -104,11 +108,20 @@ bool ResourceManager::load(const char *filename) {
 */
 bool ResourceManager::save(const char *filename, const char *mergefilename) {
     assert(filename);
-    
+
+    // these must be local variables; otherwise, the memory gets released by
+    // std::string, causing weird issues
+    string file_str = StringUtil::expandFilename(filename), mergefile_str;
+    filename = file_str.c_str();
+    if (mergefilename) {
+        mergefile_str = StringUtil::expandFilename(mergefilename);
+        mergefilename = mergefile_str.c_str();
+    }
+
     // empty database
     XrmDatabaseHelper database;
 
-    string rc_string;	
+    string rc_string;
     ResourceList::iterator i = m_resourcelist.begin();
     ResourceList::iterator i_end = m_resourcelist.end();
     //write all resources to database
@@ -125,7 +138,7 @@ bool ResourceManager::save(const char *filename, const char *mergefilename) {
     if (mergefilename) {
         // force reload of file
         m_filename = mergefilename;
-        if (m_database) 
+        if (m_database)
             delete m_database;
         m_database = 0;
 
@@ -149,40 +162,45 @@ bool ResourceManager::save(const char *filename, const char *mergefilename) {
     return true;
 }
 
-Resource_base *ResourceManager::findResource(const std::string &resname) {
+Resource_base *ResourceManager::findResource(const string &resname) {
    // find resource name
     ResourceList::iterator i = m_resourcelist.begin();
     ResourceList::iterator i_end = m_resourcelist.end();
     for (; i != i_end; ++i) {
         if ((*i)->name() == resname ||
-            (*i)->altName() == resname) 
+            (*i)->altName() == resname)
             return *i;
     }
     return 0;
 }
 
-string ResourceManager::resourceValue(const std::string &resname) {
-    Resource_base *res = findResource(resname);
+const Resource_base *ResourceManager::findResource(const string &resname) const {
+   // find resource name
+    ResourceList::const_iterator i = m_resourcelist.begin();
+    ResourceList::const_iterator i_end = m_resourcelist.end();
+    for (; i != i_end; ++i) {
+        if ((*i)->name() == resname ||
+            (*i)->altName() == resname)
+            return *i;
+    }
+    return 0;
+}
+
+string ResourceManager::resourceValue(const string &resname) const {
+    const Resource_base *res = findResource(resname);
     if (res != 0)
         return res->getString();
 
     return "";
 }
 
-void ResourceManager::setResourceValue(const std::string &resname, const std::string &value) {
+void ResourceManager::setResourceValue(const string &resname, const string &value) {
     Resource_base *res = findResource(resname);
     if (res != 0)
         res->setFromString(value.c_str());
 
 }
 
-void ResourceManager::ensureXrmIsInitialize() {
-    if (!m_init) {
-        XrmInitialize();
-        m_init = true;
-    }
-}
-
 ResourceManager &ResourceManager::lock() {
     ++m_db_lock;
     // if the lock was zero, then load the database
@@ -208,4 +226,4 @@ void ResourceManager::unlock() {
     }
 }
 
-}; // end namespace FbTk
+} // end namespace FbTk
