@@ -21,6 +21,7 @@
 
 #include "StringUtil.hh"
 
+#include "../defaults.hh"
 
 #ifdef HAVE_CSTDIO
   #include <cstdio>
@@ -48,22 +49,117 @@
   #include <string.h>
 #endif
 
+#ifdef HAVE_CERRNO
+  #include <cerrno>
+#else
+  #include <errno.h>
+#endif
 
 #include <memory>
 #include <algorithm>
 #include <string>
+#include <iostream>
 
 using std::string;
 using std::transform;
 
+namespace {
+
+template <typename T>
+int extractBigNumber(const char* in, T (*extractFunc)(const char*, char**, int), T& out) {
+
+    errno = 0;
+
+    int ret = 0;
+    char* end = 0;
+    T result = extractFunc(in, &end, 0);
+
+    if (errno == 0 && end != in) {
+        out = result;
+        ret = 1;
+    }
+
+    return ret;
+}
+
+template<typename T>
+int extractSignedNumber(const std::string& in, T& out) {
+
+    long long int result = 0;
+
+    if (::extractBigNumber(in.c_str(), strtoll, result)) {
+        out = static_cast<T>(result);
+        return 1;
+    }
+
+    return 0;
+}
+
+template<typename T>
+int extractUnsignedNumber(const std::string& in, T& out) {
+
+    unsigned long long int result = 0;
+
+    if (::extractBigNumber(in.c_str(), strtoull, result)) {
+        out = static_cast<T>(result);
+        return 1;
+    }
+
+    return 0;
+}
+
+
+
+}
+
+
 namespace FbTk {
 
 namespace StringUtil {
 
+int extractNumber(const std::string& in, int& out) {
+    return ::extractSignedNumber<int>(in, out);
+}
+
+int extractNumber(const std::string& in, unsigned int& out) {
+    return ::extractUnsignedNumber<unsigned int>(in, out);
+}
+
+int extractNumber(const std::string& in, long& out) {
+    return ::extractSignedNumber<long>(in, out);
+}
+
+int extractNumber(const std::string& in, unsigned long& out) {
+    return ::extractUnsignedNumber<unsigned long>(in, out);
+}
+
+int extractNumber(const std::string& in, long long& out) {
+    return ::extractSignedNumber<long long>(in, out);
+}
+
+int extractNumber(const std::string& in, unsigned long long& out) {
+    return ::extractUnsignedNumber<unsigned long long>(in, out);
+}
+
+
+
+std::string number2String(long long num) {
+    char s[128];
+    snprintf(s, sizeof(s), "%lld", num);
+    return std::string(s);
+}
+
+std::string number2HexString(long long num) {
+    char s[17];
+    snprintf(s, sizeof(s), "%lx", num);
+    return std::string(s);
+}
+
+
 /**
    Tries to find a string in another and
    ignoring the case of the characters
-   Returns 0 on success else pointer to str.
+   Returns 0 on failure else pointer to str.
 */
 const char *strcasestr(const char *str, const char *ptn) {
     const char *s2, *p2;
@@ -80,21 +176,89 @@ const char *strcasestr(const char *str, const char *ptn) {
     return 0;
 }
 
+
+#ifdef _WIN32
+
+#include <string>
+#define WIN32_LEAN_AND_MEAN 1
+#define NOMINMAX
+#include <windows.h>
+
+static void removeTrailingPathSeparators(std::string & path) {
+        // Remove any trailing path separators
+        size_t beforeLastPathSep = path.find_last_not_of("/\\");
+        if (beforeLastPathSep != path.size() - 1) {
+            path.erase(beforeLastPathSep + 1);
+        }
+}
+
+static std::string getFluxboxPrefix() {
+    static std::string ret;
+    static bool init = false;
+    if (!init) {
+        char buffer[1024];
+        HMODULE module = GetModuleHandle(NULL);
+        DWORD size = GetModuleFileName(module, buffer, sizeof(buffer));
+        if (sizeof(buffer) > 0)
+        {
+          buffer[sizeof(buffer) - 1] = 0;
+        }
+        static const char slash = '/';
+        static const char backslash = '\\';
+        char * lastslash = std::find_end(buffer, buffer+size, &slash, &slash + 1);
+        char * lastbackslash = std::find_end(buffer, buffer+size, &backslash, &backslash + 1);
+        ret.assign(buffer);
+
+        // Remove the filename
+        size_t lastPathSep = ret.find_last_of("/\\");
+        if (lastPathSep != std::string::npos) {
+            ret.erase(lastPathSep);
+        }
+
+        removeTrailingPathSeparators(ret);
+
+        // If the last directory is bin, remove that too.
+        lastPathSep = ret.find_last_of("/\\");
+        if (lastPathSep != std::string::npos && ret.substr(lastPathSep + 1) == "bin") {
+            ret.erase(lastPathSep);
+        }
+
+        removeTrailingPathSeparators(ret);
+    }
+    return ret;
+}
+
+#endif // _WIN32
+
 /**
    if ~ then expand it to home of user
+   if /DUMMYPREFIX on Windows then expand it to the prefix relative to the
+   executable on Windows.
    returns expanded filename
 */
 string expandFilename(const string &filename) {
     string retval;
     size_t pos = filename.find_first_not_of(" \t");
     if (pos != string::npos && filename[pos] == '~') {
+#ifdef _WIN32
+        retval = getenv("USERPROFILE");
+#else
         retval = getenv("HOME");
-        if (pos != filename.size()) {
+#endif
+        if (pos + 1 < filename.size()) {
             // copy from the character after '~'
             retval += static_cast<const char *>(filename.c_str() + pos + 1);
         }
-    } else
-        return filename; //return unmodified value
+    } else {
+        retval = filename; //return unmodified value
+    }
+
+#if defined(_WIN32) && defined(DUMMYPREFIX)
+    if (retval.find(DUMMYPREFIX) == 0) {
+      static const std::string dummyPrefix = DUMMYPREFIX;
+      retval.replace(0, dummyPrefix.size(), getFluxboxPrefix());
+    }
+#endif
 
     return retval;
 }
@@ -111,6 +275,24 @@ string findExtension(const string &filename) {
     return filename.substr(start_pos + 1);
 }
 
+string::size_type findCharFromAlphabetAfterTrigger(const std::string& in, char trigger, const char alphabet[], size_t len_alphabet, size_t* found) {
+    for (const char* s = in.c_str(); *s != '\0'; ) {
+        if (*s++ == trigger && *s != '\0') {
+            for (const char* a = alphabet; (a - alphabet) < static_cast<ssize_t>(len_alphabet); ++a) {
+                if (*s == *a) {
+                    if (found) {
+                        *found = a - alphabet;
+                    }
+                    return s - in.c_str() - 1;
+                }
+            }
+            s++;
+        }
+    }
+    return string::npos;
+}
+
+
 string replaceString(const string &original,
                      const char *findthis,
                      const char *replace) {
@@ -142,7 +324,7 @@ string replaceString(const string &original,
    for the position + 1 in the in-string where the "last"-char value
    was found.
 */
-int getStringBetween(string& out, const char *instr, const char first, const char last,
+int getStringBetween(string& out, const char *instr, char first, char last,
                      const char *ok_chars, bool allow_nesting) {
     assert(first);
     assert(last);
