@@ -1,5 +1,5 @@
 // Theme.cc for FbTk - Fluxbox ToolKit
-// Copyright (c) 2002 - 2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,8 +19,6 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Theme.cc 3782 2004-12-18 01:29:22Z mathias $
-
 #include "Theme.hh"
 
 #include "XrmDatabaseHelper.hh"
@@ -29,6 +27,7 @@
 #include "FileUtil.hh"
 #include "I18n.hh"
 #include "Image.hh"
+#include "STLUtil.hh"
 
 #ifdef HAVE_CSTDIO
   #include <cstdio>
@@ -37,11 +36,33 @@
 #endif
 #include <memory>
 #include <iostream>
+#include <algorithm>
 
-using namespace std;
+using std::cerr;
+using std::endl;
+using std::string;
 
 namespace FbTk {
 
+struct LoadThemeHelper {
+    LoadThemeHelper():m_tm(ThemeManager::instance()) {}
+    void operator ()(Theme *tm) {
+        m_tm.loadTheme(*tm);
+    }
+    void operator ()(ThemeManager::ThemeList &tmlist) {
+
+        STLUtil::forAll(tmlist, *this);
+        // send reconfiguration signal to theme and listeners
+        ThemeManager::ThemeList::iterator it = tmlist.begin();
+        ThemeManager::ThemeList::iterator it_end = tmlist.end();
+        for (; it != it_end; ++it) {
+            (*it)->reconfigSig().emit();
+        }
+    }
+
+    ThemeManager &m_tm;
+};
+
 Theme::Theme(int screen_num):m_screen_num(screen_num) {
     ThemeManager::instance().registerTheme(*this);
 }
@@ -56,38 +77,51 @@ ThemeManager &ThemeManager::instance() {
 }
 
 ThemeManager::ThemeManager():
-    // max_screens: we initialize this later so we can set m_verbose 
+    // max_screens: we initialize this later so we can set m_verbose
     // without having a display connection
-    m_max_screens(-1), 
+    m_max_screens(-1),
     m_verbose(false),
     m_themelocation("") {
 
 }
 
 bool ThemeManager::registerTheme(Theme &tm) {
-    if (m_max_screens < 0)
+    if (m_max_screens < 0) {
         m_max_screens = ScreenCount(FbTk::App::instance()->display());
+        m_themes.resize(m_max_screens);
+    }
 
     // valid screen num?
     if (m_max_screens < tm.screenNum() || tm.screenNum() < 0)
         return false;
-    // TODO: use find and return false if it's already there
-    // instead of unique 
-    m_themelist.push_back(&tm);
-    m_themelist.unique(); 
-    return true;
+
+    ThemeList::const_iterator it = m_themes[tm.screenNum()].begin(),
+                              it_end = m_themes[tm.screenNum()].end();
+    if (std::find(it, it_end, &tm) == it_end) {
+        m_themes[tm.screenNum()].push_back(&tm);
+        return true;
+    }
+    return false;
 }
 
 bool ThemeManager::unregisterTheme(Theme &tm) {
-    m_themelist.remove(&tm);
+    if (m_max_screens < tm.screenNum() || tm.screenNum() < 0)
+        return false;
+
+    m_themes[tm.screenNum()].remove(&tm);
+
     return true;
 }
 
-bool ThemeManager::load(const std::string &filename, int screen_num) {
-    std::string location = FbTk::StringUtil::expandFilename(filename);
-    std::string prefix = "";
+bool ThemeManager::load(const string &filename,
+                        const string &overlay_filename, int screen_num) {
+    
+    string location = FbTk::StringUtil::expandFilename(filename);
+    StringUtil::removeTrailingWhitespace(location);
+    StringUtil::removeFirstWhitespace(location);
+    string prefix = "";
 
-    if (FileUtil::isDirectory(filename.c_str())) {
+    if (FileUtil::isDirectory(location.c_str())) {
         prefix = location;
 
         location.append("/theme.cfg");
@@ -107,6 +141,20 @@ bool ThemeManager::load(const std::string &filename, int screen_num) {
     if (!m_database.load(location.c_str()))
         return false;
 
+
+    if (!overlay_filename.empty()) {
+        string overlay_location = FbTk::StringUtil::expandFilename(overlay_filename);
+        if (FileUtil::isRegularFile(overlay_location.c_str())) {
+            XrmDatabaseHelper overlay_db;
+            if (overlay_db.load(overlay_location.c_str())) {
+                // after a merge the src_db is destroyed
+                // so, make sure XrmDatabaseHelper::m_database == 0
+                XrmMergeDatabases(*overlay_db, &(*m_database));
+                *overlay_db = 0;
+            }
+        }
+    }
+
     // relies on the fact that load_rc clears search paths each time
     if (m_themelocation != "") {
         Image::removeSearchPath(m_themelocation);
@@ -121,29 +169,22 @@ bool ThemeManager::load(const std::string &filename, int screen_num) {
     location.append("/pixmaps");
     Image::addSearchPath(location);
 
+    LoadThemeHelper load_theme_helper;
+
     // get list and go throu all the resources and load them
-    ThemeList::iterator theme_it = m_themelist.begin();
-    const ThemeList::iterator theme_it_end = m_themelist.end();
-    for (; theme_it != theme_it_end; ++theme_it) {
-        if (screen_num < 0)
-            loadTheme(**theme_it);
-        else if (screen_num == (*theme_it)->screenNum()) // specified screen
-            loadTheme(**theme_it);
-            
-    }
-    // notify all themes that we reconfigured
-    theme_it = m_themelist.begin();
-    for (; theme_it != theme_it_end; ++theme_it) {
-        // send reconfiguration signal to theme and listeners
-        (*theme_it)->reconfigTheme();
-        (*theme_it)->reconfigSig().notify();
+    // and then reconfigure them
+    if (screen_num < 0 || screen_num > m_max_screens) {
+        STLUtil::forAll(m_themes, load_theme_helper);
+    } else {
+        load_theme_helper(m_themes[screen_num]);
     }
+
     return true;
 }
 
 void ThemeManager::loadTheme(Theme &tm) {
-    std::list<ThemeItem_base *>::iterator i = tm.itemList().begin();
-    std::list<ThemeItem_base *>::iterator i_end = tm.itemList().end();
+    Theme::ItemList::iterator i = tm.itemList().begin();
+    Theme::ItemList::iterator i_end = tm.itemList().end();
     for (; i != i_end; ++i) {
         ThemeItem_base *resource = *i;
         if (!loadItem(*resource)) {
@@ -151,7 +192,7 @@ void ThemeManager::loadTheme(Theme &tm) {
             if (!tm.fallback(*resource)) {
                 if (verbose()) {
                     _FB_USES_NLS;
-                    cerr<<_FBTKTEXT(Error, ThemeItem, "Failed to read theme item", "When reading a style, couldn't read a specific item (following)")<<": "<<resource->name()<<endl;
+                    cerr<<_FBTK_CONSOLETEXT(Error, ThemeItem, "Failed to read theme item", "When reading a style, couldn't read a specific item (following)")<<": "<<resource->name()<<endl;
                 }
                 resource->setDefaultValue();
             }
@@ -165,7 +206,7 @@ bool ThemeManager::loadItem(ThemeItem_base &resource) {
 }
 
 /// handles resource item loading with specific name/altname
-bool ThemeManager::loadItem(ThemeItem_base &resource, const std::string &name, const std::string &alt_name) {
+bool ThemeManager::loadItem(ThemeItem_base &resource, const string &name, const string &alt_name) {
     XrmValue value;
     char *value_type;
     if (XrmGetResource(*m_database, name.c_str(),
@@ -178,7 +219,7 @@ bool ThemeManager::loadItem(ThemeItem_base &resource, const std::string &name, c
     return true;
 }
 
-std::string ThemeManager::resourceValue(const std::string &name, const std::string &altname) {
+string ThemeManager::resourceValue(const string &name, const string &altname) {
     XrmValue value;
     char *value_type;
     if (*m_database != 0 && XrmGetResource(*m_database, name.c_str(),
@@ -193,10 +234,10 @@ void ThemeManager::listItems() {
     ThemeList::iterator it = m_themelist.begin();
     ThemeList::iterator it_end = m_themelist.end();
     for (; it != it_end; ++it) {
-        std::list<ThemeItem_base *>::iterator item = (*it)->itemList().begin();
-        std::list<ThemeItem_base *>::iterator item_end = (*it)->itemList().end();
+        list<ThemeItem_base *>::iterator item = (*it)->itemList().begin();
+        list<ThemeItem_base *>::iterator item_end = (*it)->itemList().end();
         for (; item != item_end; ++item) {
-            
+
             if (typeid(**item) == typeid(ThemeItem<Texture>)) {
                 cerr<<(*item)->name()<<": <texture type>"<<endl;
                 cerr<<(*item)->name()<<".pixmap:  <filename>"<<endl;
@@ -210,7 +251,7 @@ void ThemeManager::listItems() {
                 cerr<<(*item)->name()<<": <boolean>"<<endl;
             } else if (typeid(**item) == typeid(ThemeItem<PixmapWithMask>)) {
                 cerr<<(*item)->name()<<": <filename>"<<endl;
-            }  else if (typeid(**item) == typeid(ThemeItem<std::string>)) {
+            }  else if (typeid(**item) == typeid(ThemeItem<string>)) {
                 cerr<<(*item)->name()<<": <string>"<<endl;
             } else if (typeid(**item) == typeid(ThemeItem<Font>)) {
                 cerr<<(*item)->name()<<": <font>"<<endl;
@@ -219,7 +260,7 @@ void ThemeManager::listItems() {
             }
         }
     }
-             
+
 }
 */
-}; // end namespace FbTk
+} // end namespace FbTk
