@@ -1,5 +1,5 @@
 // ThemeItems.cc for FbTk - Fluxbox ToolKit
-// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,9 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: ThemeItems.cc 4107 2005-09-14 19:22:59Z fluxgen $
-
-/// @file implements common theme items
+/// @file ThemeItems.cc implements common theme items
 
 #ifndef THEMEITEMS_HH
 #define THEMEITEMS_HH
@@ -37,6 +35,7 @@
 #include "GContext.hh"
 #include "PixmapWithMask.hh"
 #include "Image.hh"
+#include "Shape.hh"
 #include "StringUtil.hh"
 
 #include <string>
@@ -45,6 +44,16 @@
 #else
   #include <stdio.h>
 #endif
+#ifdef HAVE_CSTRING
+  #include <cstring>
+#else
+  #include <string.h>
+#endif
+#ifdef HAVE_CSTDLIB
+  #include <cstdlib>
+#else
+  #include <stdlib.h>
+#endif
 
 #include <iostream>
 #include <memory>
@@ -71,7 +80,7 @@ void ThemeItem<string>::setFromString(const char *str) {
 }
 
 template <>
-void ThemeItem<int>::load(const string *name, const string *altname) { }
+void ThemeItem<int>::load(const std::string *name, const std::string *altname) { }
 
 template<>
 void ThemeItem<int>::setDefaultValue() {
@@ -89,15 +98,15 @@ void ThemeItem<int>::setFromString(const char *str) {
         setDefaultValue();
 }
 template<>
-void FbTk::ThemeItem<bool>::load(const std::string *name, const std::string *altname) { }
+void ThemeItem<bool>::load(const std::string *name, const std::string *altname) { }
 
 template<>
-void FbTk::ThemeItem<bool>::setDefaultValue() {
+void ThemeItem<bool>::setDefaultValue() {
     *(*this) = false;
 }
 
 template<>
-void FbTk::ThemeItem<bool>::setFromString(char const *strval) {
+void ThemeItem<bool>::setFromString(char const *strval) {
     if (strcasecmp(strval, "true")==0)
         *(*this) = true;
     else
@@ -111,7 +120,7 @@ void ThemeItem<unsigned int>::setDefaultValue() {
 
 template <>
 void ThemeItem<unsigned int>::setFromString(const char *str) {
-    sscanf(str, "%d", &m_value);
+    sscanf(str, "%u", &m_value);
 }
 
 template <>
@@ -120,25 +129,11 @@ void ThemeItem<unsigned int>::load(const std::string *name, const std::string *a
 
 template <>
 void ThemeItem<Font>::setDefaultValue() {
-    if (!m_value.load("fixed")) {
+    if (!m_value.load("__DEFAULT__")) {
         cerr<<"ThemeItem<Font>: Warning! Failed to load default value 'fixed'"<<endl;
     } else {
-        string effect(ThemeManager::instance().resourceValue(name()+".effect", altName()+".Effect"));
-        if (effect == "halo") {
-            m_value.setHalo(true);
-            FbTk::Color halo_color(ThemeManager::instance().resourceValue(name()+".halo.color", altName()+".Halo.Color").c_str(), 
-                    theme().screenNum());
-            m_value.setHaloColor(halo_color);
-
-        } else if (effect == "shadow" ) {
-            FbTk::Color shadow_color(ThemeManager::instance().resourceValue(name()+".shadow.color", altName()+".Shadow.Color").c_str(), 
-                    theme().screenNum());
-            
-            m_value.setShadow(true);
-            m_value.setShadowColor(shadow_color);
-            m_value.setShadowOffX(atoi(ThemeManager::instance().resourceValue(name()+".shadow.x", altName()+".Shadow.X").c_str()));
-            m_value.setShadowOffY(atoi(ThemeManager::instance().resourceValue(name()+".shadow.y", altName()+".Shadow.Y").c_str()));
-        }
+        m_value.setHalo(false);
+        m_value.setShadow(false);
     }
 }
 
@@ -154,34 +149,30 @@ void ThemeItem<Font>::setFromString(const char *str) {
             cerr<<"Theme: Setting default value"<<endl;
         }
         setDefaultValue();
-    } else {
-        string effect(ThemeManager::instance().resourceValue(name()+".effect", altName()+".Effect"));
-        if (effect == "halo") {
-            m_value.setHalo(true);
-            FbTk::Color halo_color(ThemeManager::instance().resourceValue(name()+".halo.color", altName()+".Halo.Color").c_str(), 
-                    theme().screenNum());
-            m_value.setHaloColor(halo_color);
-
-        } else if (effect == "shadow" ) {
-            FbTk::Color shadow_color(ThemeManager::instance().resourceValue(name()+".shadow.color", altName()+".Shadow.Color").c_str(), 
-                    theme().screenNum());
-            
-            m_value.setShadow(true);
-            m_value.setShadowColor(shadow_color);
-
-            int offset_x = atoi(ThemeManager::instance().resourceValue(name()+".shadow.x", altName()+".Shadow.X").c_str());
-            int offset_y = atoi(ThemeManager::instance().resourceValue(name()+".shadow.y", altName()+".Shadow.Y").c_str());
-            if (offset_x != 0)
-                m_value.setShadowOffX(offset_x);
-            if (offset_y != 0)
-                m_value.setShadowOffY(offset_y);
-        }
     }
 }
 
-// do nothing
 template <>
-void ThemeItem<Font>::load(const string *name, const string *altname) {
+void ThemeItem<Font>::load(const string *o_name, const string *o_altname) {
+    const string &m_name = o_name ? *o_name : name();
+    const string &m_altname = o_altname ? *o_altname : altName();
+
+    string effect(ThemeManager::instance().resourceValue(m_name+".effect", m_altname+".Effect"));
+    if (effect == "halo") {
+        Color halo_color(ThemeManager::instance().resourceValue(m_name+".halo.color", m_altname+".Halo.Color").c_str(), 
+                theme().screenNum());
+
+        m_value.setHalo(true);
+        m_value.setHaloColor(halo_color);
+    } else if (effect == "shadow" ) {
+        Color shadow_color(ThemeManager::instance().resourceValue(m_name+".shadow.color", m_altname+".Shadow.Color").c_str(), 
+                theme().screenNum());
+        
+        m_value.setShadow(true);
+        m_value.setShadowColor(shadow_color);
+        m_value.setShadowOffX(atoi(ThemeManager::instance().resourceValue(m_name+".shadow.x", m_altname+".Shadow.X").c_str()));
+        m_value.setShadowOffY(atoi(ThemeManager::instance().resourceValue(m_name+".shadow.y", m_altname+".Shadow.Y").c_str()));
+    }
 }
 
 
@@ -218,8 +209,8 @@ void ThemeItem<Texture>::load(const string *o_name, const string *o_altname) {
         return;
     }
 
-    std::auto_ptr<PixmapWithMask> pm(Image::load(pixmap_name,
-                                                             m_tm.screenNum()));
+    std::auto_ptr<PixmapWithMask> pm(Image::load(pixmap_name, m_tm.screenNum()));
+
     if (pm.get() == 0) {
         if (ThemeManager::instance().verbose()) {
             cerr<<"Resource("<<m_name+".pixmap"
@@ -233,7 +224,7 @@ void ThemeItem<Texture>::load(const string *o_name, const string *o_altname) {
 
 template <>
 void ThemeItem<Texture>::setDefaultValue() {
-    m_value.setType(Texture::FLAT | Texture::SOLID);
+    m_value.setType(Texture::DEFAULT_LEVEL | Texture::DEFAULT_TEXTURE);
     load(); // one might forget to add line something:  so we try to load something.*:  too
 }
 
@@ -363,7 +354,29 @@ void ThemeItem<GContext::CapStyle>::setFromString(char const *strval) {
 template<>
 void ThemeItem<GContext::CapStyle>::load(const string *name, const string *altname) { }
 
+template <>
+void ThemeItem<Shape::ShapePlace>::load(const string *name, const string *altname) { }
 
+template <>
+void ThemeItem<Shape::ShapePlace>::setDefaultValue() {
+    *(*this) = Shape::NONE;
+}
+
+template <>
+void ThemeItem<Shape::ShapePlace>::setFromString(const char *str) {
+    int places = 0;
+
+    if (StringUtil::strcasestr(str, "topleft") != 0)
+        places |= Shape::TOPLEFT;
+    if (StringUtil::strcasestr(str, "topright") != 0)
+        places |= Shape::TOPRIGHT;
+    if (StringUtil::strcasestr(str, "bottomleft") != 0)
+        places |= Shape::BOTTOMLEFT;
+    if (StringUtil::strcasestr(str, "bottomright") != 0)
+        places |= Shape::BOTTOMRIGHT;
+
+    *(*this) = static_cast<Shape::ShapePlace>(places);
+}
 
 } // end namespace FbTk
 
