@@ -1,5 +1,5 @@
 // XftFontImp.cc  Xft font implementation for FbTk
-// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,27 +19,34 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-//$Id: XftFontImp.cc 4043 2005-06-03 07:25:48Z mathias $
-
 #include "XftFontImp.hh"
 #include "App.hh"
 #include "FbDrawable.hh"
 
+#include <math.h>
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif //HAVE_CONFIG_H
 
 namespace FbTk {
 
-XftFontImp::XftFontImp(const char *name, bool utf8):m_xftfont(0),
-                                                    m_utf8mode(utf8) {
+XftFontImp::XftFontImp(const char *name, bool utf8):
+    m_utf8mode(utf8), m_name("") {
+
+    for (int r = ROT0; r <= ROT270; r++) {
+        m_xftfonts[r] = 0;
+        m_xftfonts_loaded[r] = false;
+    }
+
     if (name != 0)
         load(name);
 }
 
 XftFontImp::~XftFontImp() {
-    if (m_xftfont != 0)
-        XftFontClose(App::instance()->display(), m_xftfont);
+    for (int r = ROT0; r <= ROT270; r++) 
+        if (m_xftfonts[r] != 0)
+            XftFontClose(App::instance()->display(), m_xftfonts[r]);
 }
 
 bool XftFontImp::load(const std::string &name) {
@@ -54,22 +61,51 @@ bool XftFontImp::load(const std::string &name) {
             return false;
     }
     
-    // destroy old font and set new
-    if (m_xftfont != 0)
-        XftFontClose(disp, m_xftfont);
+    // destroy all old fonts and set new
+    for (int r = ROT0; r <= ROT270; r++) {
+        m_xftfonts_loaded[r] = false;
+        if (m_xftfonts[r] != 0) {
+            XftFontClose(App::instance()->display(), m_xftfonts[r]);
+            m_xftfonts[r] = 0;
+        }
+    }
 
-    m_xftfont = newxftfont;
+    m_xftfonts[ROT0] = newxftfont;
+    m_xftfonts_loaded[ROT0] = true;
+    m_name = name;
 
     return true;
 }
 
-void XftFontImp::drawText(const FbDrawable &w, int screen, GC gc, const char *text, size_t len, int x, int y) const {
-    if (m_xftfont == 0)
+void XftFontImp::drawText(const FbDrawable &w, int screen, GC gc, const char* text, size_t len, int x, int y, FbTk::Orientation orient) {
+
+    if (!text || !*text)
         return;
-    XftDraw *draw = XftDrawCreate(w.display(),
-                                  w.drawable(),
-                                  DefaultVisual(w.display(), screen),
-                                  DefaultColormap(w.display(), screen));
+
+    if (!validOrientation(orient))
+        return;
+
+    // we adjust y slightly so that the baseline is in the right spot
+    // (it is offset one by rotation >=180 degrees)
+    switch (orient) {
+    case ROT0:
+        break;
+    case ROT90:
+        break;
+    case ROT180:
+        x+=1;
+        y+=1;
+        break;
+    case ROT270:
+        y+=1;
+        break;
+    }
+
+    Visual* def_visual = DefaultVisual(w.display(), screen);
+    Colormap def_colmap = DefaultColormap(w.display(), screen);
+
+    XftFont *font = m_xftfonts[orient];
+    XftDraw *draw = XftDrawCreate(w.display(), w.drawable(), def_visual, def_colmap);
 
     XGCValues gc_val;
 
@@ -80,7 +116,7 @@ void XftFontImp::drawText(const FbDrawable &w, int screen, GC gc, const char *te
     // get red, green, blue values
     XColor xcol;
     xcol.pixel = gc_val.foreground;
-    XQueryColor(w.display(), DefaultColormap(w.display(), screen), &xcol);
+    XQueryColor(w.display(), def_colmap, &xcol);
 
     // convert xcolor to XftColor
     XRenderColor rendcol;
@@ -89,10 +125,7 @@ void XftFontImp::drawText(const FbDrawable &w, int screen, GC gc, const char *te
     rendcol.blue = xcol.blue;
     rendcol.alpha = 0xFFFF;
     XftColor xftcolor;
-    XftColorAllocValue(w.display(), 
-                       DefaultVisual(w.display(), screen), 
-                       DefaultColormap(w.display(), screen),
-                       &rendcol, &xftcolor);
+    XftColorAllocValue(w.display(), def_visual, def_colmap, &rendcol, &xftcolor);
 
     // draw string
 #ifdef HAVE_XFT_UTF8_STRING
@@ -100,49 +133,38 @@ void XftFontImp::drawText(const FbDrawable &w, int screen, GC gc, const char *te
         // check the string size,
         // if the size is zero we use the XftDrawString8 function instead.
         XGlyphInfo ginfo;
-        XftTextExtentsUtf8(w.display(),
-                           m_xftfont,
-                           (XftChar8 *)text, len,
-                           &ginfo);
+        XftTextExtentsUtf8(w.display(), m_xftfonts[ROT0], (XftChar8 *)text, len, &ginfo);
         if (ginfo.xOff != 0) {
-            XftDrawStringUtf8(draw,
-                              &xftcolor,
-                              m_xftfont,
-                              x, y,
-                              (XftChar8 *)(text), len);
-            XftColorFree(w.display(), 
-                         DefaultVisual(w.display(), screen),
-                         DefaultColormap(w.display(), screen), &xftcolor);
+            XftDrawStringUtf8(draw, &xftcolor, font, x, y, (XftChar8 *)text, len);
+            XftColorFree(w.display(), def_visual, def_colmap, &xftcolor);
             XftDrawDestroy(draw);
             return;
         }
     }
 #endif // HAVE_XFT_UTF8_STRING
 
-    XftDrawString8(draw,
-                   &xftcolor,
-                   m_xftfont,
-                   x, y,
-                   (XftChar8 *)(text), len);
+    XftDrawString8(draw, &xftcolor, font, x, y, (XftChar8 *)text, len);
 
 
-    XftColorFree(w.display(), 
-                 DefaultVisual(w.display(), screen),
-                 DefaultColormap(w.display(), screen), &xftcolor);
+    XftColorFree(w.display(), def_visual, def_colmap, &xftcolor);
     XftDrawDestroy(draw);
 }
 
-unsigned int XftFontImp::textWidth(const char * const text, unsigned int len) const {
-    if (m_xftfont == 0)
+unsigned int XftFontImp::textWidth(const char* text, unsigned int len) const {
+
+    if (m_xftfonts[ROT0] == 0)
         return 0;
 
     XGlyphInfo ginfo;
     Display* disp = App::instance()->display();
 
+    XftFont *font = m_xftfonts[ROT0];
+
+
 #ifdef HAVE_XFT_UTF8_STRING
     if (m_utf8mode) {
         XftTextExtentsUtf8(disp,
-                           m_xftfont,
+                           font,
                            (XftChar8 *)text, len,
                            &ginfo);
         if (ginfo.xOff != 0)
@@ -152,21 +174,62 @@ unsigned int XftFontImp::textWidth(const char * const text, unsigned int len) co
     }
 #endif  //HAVE_XFT_UTF8_STRING
 
+    std::string localestr = FbStringUtil::FbStrToLocale(FbString(text, len));
+
     XftTextExtents8(disp,
-                    m_xftfont,
-                    (XftChar8 *)text, len,
+                    font,
+                    (XftChar8 *)localestr.data(), localestr.size(),
                     &ginfo);
 
     return ginfo.xOff;
 }
 
 unsigned int XftFontImp::height() const {
-    if (m_xftfont == 0)
+    if (m_xftfonts[ROT0] == 0)
         return 0;
-    return m_xftfont->height;
+    else
+        return m_xftfonts[ROT0]->height;
     //m_xftfont->ascent + m_xftfont->descent;
     // curiously, fonts seem to have a smaller height, but the "height"
     // is specified within the actual font, so it must be right, right?
 }
 
-}; // end namespace FbTk
+bool XftFontImp::validOrientation(FbTk::Orientation orient) {
+    if (orient == ROT0 || m_xftfonts[orient])
+        return true;
+
+    if (m_xftfonts_loaded[orient])
+        return false; // m_xftfonts is zero here
+
+    if (m_xftfonts[ROT0] == 0)
+        return false;
+
+    m_xftfonts_loaded[orient] = true;
+
+    // otherwise, try to load that orientation
+    // radians is actually anti-clockwise, so we reverse it
+    double radians = -(orient) * 90 * M_PI / 180;
+
+    XftMatrix matrix;
+    XftMatrixInit(&matrix);
+    XftMatrixRotate(&matrix, cos(radians), sin(radians));
+
+    Display *disp = App::instance()->display();
+
+    XftPattern * pattern = XftNameParse(m_name.c_str());
+    XftResult result;
+    pattern = XftFontMatch(disp, 0, pattern, &result);
+    XftPatternAddMatrix(pattern, XFT_MATRIX, &matrix);
+    XftFont * new_font = XftFontOpenPattern(disp, pattern);
+
+    if (new_font == 0)
+        return false;
+
+    m_xftfonts[orient] = new_font;
+
+    return true;
+}
+
+
+
+} // end namespace FbTk
