@@ -1,5 +1,5 @@
 // XmbFontImp.cc for FbTk fluxbox toolkit
-// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,13 +19,13 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: XmbFontImp.cc 4042 2005-06-03 07:22:27Z mathias $
-
 #include "XmbFontImp.hh"
 
 #include "App.hh"
+#include "TextUtils.hh"
 #include "StringUtil.hh"
-#include "FbDrawable.hh"
+#include "FbPixmap.hh"
+#include "GContext.hh"
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -49,14 +49,13 @@
 #else
   #include <stdarg.h>
 #endif
-#include <iostream>
 #ifdef HAVE_CSTRING
   #include <cstring>
 #else
   #include <string.h>
 #endif
 
-using namespace std;
+using std::string;
 
 namespace {
 
@@ -113,16 +112,18 @@ const char *getFontElement(const char *pattern, char *buf, int bufsiz, ...) {
 XFontSet createFontSet(const char *fontname, bool& utf8mode) {
     Display *display = FbTk::App::instance()->display();
     XFontSet fs;
-    const int FONT_ELEMENT_SIZE=50;
-    char **missing, *def = "-";
-    int nmissing, pixel_size = 0, buf_size = 0;
-    char weight[FONT_ELEMENT_SIZE], slant[FONT_ELEMENT_SIZE];
-    std::string orig_locale = "";
+    char **missing;
+    const char *constdef = "-";
+    char *def = const_cast<char *>(constdef);
+    int nmissing;
+    string orig_locale = "";
 
 #ifdef HAVE_SETLOCALE
     if (utf8mode) {
         orig_locale = setlocale(LC_CTYPE, NULL);
-        setlocale(LC_CTYPE, "UTF-8");
+        if (setlocale(LC_CTYPE, "UTF-8") == NULL) {
+            utf8mode = false;
+        }
     }
 #endif // HAVE_SETLOCALE
     fs = XCreateFontSet(display,
@@ -145,17 +146,20 @@ XFontSet createFontSet(const char *fontname, bool& utf8mode) {
         fs = XCreateFontSet(display, fontname,
                             &missing, &nmissing, &def);
         setlocale(LC_CTYPE, orig_locale.c_str());
+        return fs;
     }
     if (utf8mode)
         setlocale(LC_CTYPE, orig_locale.c_str());
 #endif // HAVE_SETLOCALE
 
+    // set to false because our strings won't be utf8-happy
     utf8mode = false;
-    
+
     return fs;
 }
 
-};
+}
+
 namespace FbTk {
 
 XmbFontImp::XmbFontImp(const char *filename, bool utf8) : m_fontset(0), m_setextents(0), m_utf8mode(utf8) {
@@ -168,7 +172,7 @@ XmbFontImp::~XmbFontImp() {
         XFreeFontSet(App::instance()->display(), m_fontset);
 }
 
-bool XmbFontImp::load(const std::string &fontname) {
+bool XmbFontImp::load(const string &fontname) {
     if (fontname.empty())
         return false;
 
@@ -185,41 +189,99 @@ bool XmbFontImp::load(const std::string &fontname) {
     return true;
 }
 
-void XmbFontImp::drawText(const FbDrawable &w, int screen, GC gc, const char *text,
-                          size_t len, int x, int y) const {
+void XmbFontImp::drawText(const FbDrawable &d, int screen, GC main_gc, const char* text,
+                          size_t len, int x, int y, FbTk::Orientation orient) {
 
-    if (m_fontset == 0)
+    if (!text || !*text || m_fontset == 0)
         return;
 
+    if (orient == ROT0) {
+#ifdef X_HAVE_UTF8_STRING
+        if (m_utf8mode) {
+            Xutf8DrawString(d.display(), d.drawable(), m_fontset,
+                            main_gc, x, y,
+                            text, len);
+        } else
+#endif //X_HAVE_UTF8_STRING
+            {
+                std::string localestr = FbStringUtil::FbStrToLocale(FbString(text, len));
+                XmbDrawString(d.display(), d.drawable(), m_fontset,
+                              main_gc, x, y,
+                              localestr.data(), localestr.size());
+            }
+        return;
+    }
+
+    Display *dpy = App::instance()->display();
+
+    int xpos = x, ypos = y;
+    unsigned int w = d.width();
+    unsigned int h = d.height();
+
+    translateSize(orient, w, h);
+    untranslateCoords(orient, xpos, ypos, w, h);
+
+    // not straight forward, we actually draw it elsewhere, then rotate it
+    FbTk::FbPixmap canvas(d.drawable(), w, h, 1);
+
+    // create graphic context for our canvas
+    FbTk::GContext font_gc(canvas);
+    font_gc.setBackground(None);
+    font_gc.setForeground(None);
+
+    XFillRectangle(dpy, canvas.drawable(), font_gc.gc(), 0, 0, canvas.width(), canvas.height());
+    font_gc.setForeground(1);
+
+
 #ifdef X_HAVE_UTF8_STRING
     if (m_utf8mode) {
-        Xutf8DrawString(w.display(), w.drawable(), m_fontset,
-                        gc, x, y,
-                        text, len);
+        Xutf8DrawString(dpy, canvas.drawable(), m_fontset,
+                             font_gc.gc(), xpos, ypos,
+                             text, len);
     } else
 #endif //X_HAVE_UTF8_STRING
     {
-        XmbDrawString(w.display(), w.drawable(), m_fontset,
-                      gc, x, y,
-                      text, len);
+        std::string localestr = FbStringUtil::FbStrToLocale(FbString(text, len));
+        XmbDrawString(dpy, canvas.drawable(), m_fontset,
+                           font_gc.gc(), xpos, ypos,
+                           localestr.data(), localestr.size());
     }
+
+    canvas.rotate(orient);
+
+    GC my_gc = XCreateGC(dpy, d.drawable(), 0, 0);
+
+    XCopyGC(dpy, main_gc, GCForeground|GCBackground, my_gc);
+
+    // vertical or upside down
+    XSetFillStyle(dpy, my_gc, FillStippled);
+    XSetStipple(dpy, my_gc, canvas.drawable());
+    XSetTSOrigin(dpy, my_gc, 0, 0);
+
+    XFillRectangle(dpy, d.drawable(), my_gc, 0, 0,
+                   canvas.width(),
+                   canvas.height());
+
+    XFreeGC(dpy, my_gc);
+
 }
 
-unsigned int XmbFontImp::textWidth(const char * const text, unsigned int len) const {
+unsigned int XmbFontImp::textWidth(const char* text, unsigned int len) const {
+
     if (m_fontset == 0)
         return 0;
 
     XRectangle ink, logical;
 #ifdef X_HAVE_UTF8_STRING
     if (m_utf8mode) {
-        Xutf8TextExtents(m_fontset, text, len,
-                         &ink, &logical);
+        Xutf8TextExtents(m_fontset, text, len, &ink, &logical);
         if (logical.width != 0)
             return logical.width;
     }
 #endif // X_HAVE_UTF8_STRING
 
-    XmbTextExtents(m_fontset, text, len,
+    std::string localestr = FbStringUtil::FbStrToLocale(FbString(text, len));
+    XmbTextExtents(m_fontset, localestr.data(), localestr.size(),
                    &ink, &logical);
     return logical.width;
 }
@@ -230,5 +292,5 @@ unsigned int XmbFontImp::height() const {
     return m_setextents->max_ink_extent.height;
 }
 
-}; // end namespace FbTk
+} // end namespace FbTk
 
