@@ -266,7 +266,7 @@ IconbarTool::IconbarTool(const FbTk::FbWindow &parent, IconbarTheme &theme,
     m_rc_use_pixmap(screen.resourceManager(), true,
                     screen.name() + ".iconbar.usePixmap", screen.altName() + ".Iconbar.UsePixmap"),
     m_menu(screen.menuTheme(), screen.imageControl(),
-           *screen.layerManager().getLayer(Layer::MENU)),
+           *screen.layerManager().getLayer(ResourceLayer::MENU)),
     m_alpha(255) {
 
     // setup mode menu
@@ -291,10 +291,12 @@ IconbarTool::IconbarTool(const FbTk::FbWindow &parent, IconbarTheme &theme,
     menu.insert(m_menu.label().logical(), &m_menu);
 
     // setup signals
-    theme.reconfigSig().attach(this);
-    focused_theme.reconfigSig().attach(this);
-    unfocused_theme.reconfigSig().attach(this);
-    setMode(*m_rc_mode);
+    m_tracker.join(theme.reconfigSig(), FbTk::MemFun(*this, &IconbarTool::themeReconfigured));
+    m_tracker.join(focused_theme.reconfigSig(),
+            FbTk::MemFun(*this, &IconbarTool::themeReconfigured));
+    m_tracker.join(unfocused_theme.reconfigSig(),
+            FbTk::MemFun(*this, &IconbarTool::themeReconfigured));
+    themeReconfigured();
 }
 
 IconbarTool::~IconbarTool() {
@@ -307,7 +309,7 @@ void IconbarTool::move(int x, int y) {
 
 void IconbarTool::resize(unsigned int width, unsigned int height) {
     m_icon_container.resize(width, height);
-    m_icon_container.setMaxTotalSize(m_icon_container.orientation() == FbTk::ROT0 || m_icon_container.orientation() ? width : height);
+    m_icon_container.setMaxTotalSize(m_icon_container.orientation() == FbTk::ROT0 || m_icon_container.orientation() == FbTk::ROT180 ? width : height);
     renderTheme();
 }
 
@@ -315,7 +317,7 @@ void IconbarTool::moveResize(int x, int y,
                              unsigned int width, unsigned int height) {
 
     m_icon_container.moveResize(x, y, width, height);
-    m_icon_container.setMaxTotalSize(m_icon_container.orientation() == FbTk::ROT0 || m_icon_container.orientation() ? width : height);
+    m_icon_container.setMaxTotalSize(m_icon_container.orientation() == FbTk::ROT0 || m_icon_container.orientation() == FbTk::ROT180 ? width : height);
     renderTheme();
 }
 
@@ -329,7 +331,7 @@ void IconbarTool::hide() {
 
 void IconbarTool::setAlignment(FbTk::Container::Alignment align) {
     *m_rc_alignment = align;
-    update(0);
+    update(ALIGN, NULL);
     m_menu.reconfigure();
 }
 
@@ -342,22 +344,24 @@ void IconbarTool::setMode(string mode) {
     // lock graphics update
     m_icon_container.setUpdateLock(true);
 
-    if (m_winlist.get()) {
-        m_winlist->addSig().detach(this);
-        m_winlist->removeSig().detach(this);
-        m_winlist->orderSig().detach(this);
-        m_winlist->resetSig().detach(this);
-    }
     if (mode == "none")
         m_winlist.reset(new FocusableList(m_screen));
     else
         m_winlist.reset(new FocusableList(m_screen,
                                            mode + " (iconhidden=no)"));
     if (m_winlist.get()) {
-        m_winlist->addSig().attach(this);
-        m_winlist->removeSig().attach(this);
-        m_winlist->orderSig().attach(this);
-        m_winlist->resetSig().attach(this);
+        m_winlist->addSig().connect(
+                    std::bind1st(FbTk::MemFun(*this, &IconbarTool::update), LIST_ADD)
+                );
+        m_winlist->removeSig().connect(
+                    std::bind1st(FbTk::MemFun(*this, &IconbarTool::update), LIST_REMOVE)
+                );
+        m_winlist->addSig().connect(
+                    std::bind1st(FbTk::MemFun(*this, &IconbarTool::update), LIST_ORDER)
+                );
+        m_winlist->resetSig().connect(FbTk::MemFunBind(
+                        *this, &IconbarTool::update, LIST_RESET, static_cast<Focusable *>(0)
+                    ));
     }
     reset();
 
@@ -383,7 +387,11 @@ unsigned int IconbarTool::borderWidth() const {
     return m_icon_container.borderWidth();
 }
 
-void IconbarTool::update(FbTk::Subject *subj) {
+void IconbarTool::themeReconfigured() {
+    setMode(*m_rc_mode);
+}
+
+void IconbarTool::update(UpdateReason reason, Focusable *win) {
     // ignore updates if we're shutting down
     if (m_screen.isShuttingdown()) {
         if (!m_icons.empty())
@@ -396,27 +404,21 @@ void IconbarTool::update(FbTk::Subject *subj) {
     *m_rc_client_width = FbTk::Util::clamp(*m_rc_client_width, 10, 400);
     m_icon_container.setMaxSizePerClient(*m_rc_client_width);
 
-    if (subj == &m_focused_theme.reconfigSig() ||
-        subj == &m_unfocused_theme.reconfigSig() ||
-        subj == &m_theme.reconfigSig()) {
-        setMode(*m_rc_mode);
-        return;
-    }
-
     // lock graphic update
     m_icon_container.setUpdateLock(true);
 
-    if (subj && typeid(*subj) == typeid(FocusableList::FocusableListSubject)) {
-        FocusableList::FocusableListSubject *fsubj =
-            static_cast<FocusableList::FocusableListSubject *>(subj);
-        if (subj == &m_winlist->addSig())
-            insertWindow(*fsubj->win());
-        else if (subj == &m_winlist->removeSig())
-            removeWindow(*fsubj->win());
-        else if (subj == &m_winlist->resetSig())
+    switch(reason) {
+        case LIST_ADD: case LIST_ORDER:
+            insertWindow(*win);
+            break;
+        case LIST_REMOVE:
+            removeWindow(*win);
+            break;
+        case LIST_RESET:
             reset();
-        else if (subj == &m_winlist->orderSig())
-            insertWindow(*fsubj->win());
+            break;
+        case ALIGN:
+            break;
     }
 
     // unlock container and update graphics
