@@ -19,25 +19,26 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: RootCmdMenuItem.cc 4199 2006-02-16 06:53:05Z mathias $
-
 #include "RootCmdMenuItem.hh"
 
+#include "defaults.hh"
 #include "FbCommands.hh"
 #include "fluxbox.hh"
 
 #include "FbTk/StringUtil.hh"
 
-RootCmdMenuItem::RootCmdMenuItem(const std::string &label, 
+RootCmdMenuItem::RootCmdMenuItem(const FbTk::FbString &label, 
                              const std::string &filename,
                              const std::string &cmd):
-    FbTk::MenuItem(label.c_str()), 
+    FbTk::MenuItem(label), 
     m_filename(filename) {
-    
-      FbTk::RefCount<FbTk::Command> 
-        setwp_cmd(new FbCommands::ExecuteCmd(cmd + " \"" + m_filename + "\""));
+
+    std::string prog = cmd.empty() ? realProgramName("fbsetbg") : cmd;
+    FbTk::RefCount<FbTk::Command<void> >
+        setwp_cmd(new FbCommands::ExecuteCmd(prog + " \"" + m_filename + "\""));
     setCommand(setwp_cmd);
     setToggleItem(true);
+    setCloseOnClick(false);
 }
 
 
