@@ -55,7 +55,9 @@
 #include "HeadArea.hh"
 #include "RectangleUtil.hh"
 #include "FbCommands.hh"
+#ifdef USE_SYSTRAY
 #include "SystemTray.hh"
+#endif
 #include "Debug.hh"
 
 #include "FbTk/I18n.hh"
@@ -84,13 +86,13 @@
 #include "config.h"
 #endif // HAVE_CONFIG_H
 
-#ifdef SLIT
+#ifdef USE_SLIT
 #include "Slit.hh"
 #include "SlitClient.hh"
 #else
 // fill it in
 class Slit {};
-#endif // SLIT
+#endif // USE_SLIT
 
 #ifdef USE_TOOLBAR
 #include "Toolbar.hh"
@@ -132,7 +134,7 @@ extern  "C" {
 }
 #endif // XINERAMA
 
-#ifdef HAVE_RANDR
+#if defined(HAVE_RANDR) || defined(HAVE_RANDR1_2)
 #include <X11/extensions/Xrandr.h>
 #endif // HAVE_RANDR
 
@@ -229,6 +231,22 @@ const TabPlacementString placement_strings[] = {
     { FbWinFrame::RIGHTTOP, "RightTop" }
 };
 
+Atom atom_fbcmd = 0;
+Atom atom_wm_check = 0;
+Atom atom_net_desktop = 0;
+Atom atom_utf8_string = 0;
+Atom atom_kde_systray = 0;
+Atom atom_kwm1 = 0;
+
+void initAtoms(Display* dpy) {
+    atom_wm_check = XInternAtom(dpy, "_NET_SUPPORTING_WM_CHECK", False);
+    atom_net_desktop = XInternAtom(dpy, "_NET_CURRENT_DESKTOP", False);
+    atom_fbcmd = XInternAtom(dpy, "_FLUXBOX_ACTION", False);
+    atom_utf8_string = XInternAtom(dpy, "UTF8_STRING", False);
+    atom_kde_systray = XInternAtom(dpy, "_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR", False);
+    atom_kwm1 = XInternAtom(dpy, "KWM_DOCKWINDOW", False);
+}
+
 
 } // end anonymous namespace
 
@@ -315,8 +333,7 @@ BScreen::BScreen(FbTk::ResourceManager &rm,
     m_geom_window(new OSDWindow(m_root_window, *this, *m_focused_windowtheme)),
     m_pos_window(new OSDWindow(m_root_window, *this, *m_focused_windowtheme)),
     m_tooltip_window(new TooltipWindow(m_root_window, *this, *m_focused_windowtheme)),
-    m_dummy_window(scrn, -1, -1, 1, 1, 0, true, false, CopyFromParent,
-                   InputOnly),
+    m_dummy_window(scrn, -1, -1, 1, 1, 0, true, false, CopyFromParent, InputOnly),
     resource(rm, screenname, altscreenname),
     m_resource_manager(rm),
     m_name(screenname),
@@ -329,8 +346,11 @@ BScreen::BScreen(FbTk::ResourceManager &rm,
     m_shutdown(false) {
 
 
-    Display *disp = m_root_window.display();
     Fluxbox *fluxbox = Fluxbox::instance();
+    Display *disp = fluxbox->display();
+
+    initAtoms(disp);
+
 
     // TODO fluxgen: check if this is the right place (it was not -lis)
     //
@@ -347,7 +367,7 @@ BScreen::BScreen(FbTk::ResourceManager &rm,
                               SubstructureRedirectMask | KeyPressMask | KeyReleaseMask |
                               ButtonPressMask | ButtonReleaseMask| SubstructureNotifyMask);
 
-    FbTk::App::instance()->sync(false);
+    fluxbox->sync(false);
 
     XSetErrorHandler((XErrorHandler) old);
 
@@ -368,31 +388,37 @@ BScreen::BScreen(FbTk::ResourceManager &rm,
 #endif // HAVE_GETPID
 
     // check if we're the first EWMH compliant window manager on this screen
-    Atom wm_check = XInternAtom(disp, "_NET_SUPPORTING_WM_CHECK", False);
     Atom xa_ret_type;
     int ret_format;
     unsigned long ret_nitems, ret_bytes_after;
     unsigned char *ret_prop;
-    if (rootWindow().property(wm_check, 0l, 1l,
+    if (rootWindow().property(atom_wm_check, 0l, 1l,
             False, XA_WINDOW, &xa_ret_type, &ret_format, &ret_nitems,
             &ret_bytes_after, &ret_prop) ) {
         m_restart = (ret_prop != NULL);
         XFree(ret_prop);
     }
 
-#ifdef HAVE_RANDR
-    // setup RANDR for this screens root window
-    // we need to determine if we should use old randr select input function or not
-#ifdef X_RRScreenChangeSelectInput
-    // use old set randr event
-    XRRScreenChangeSelectInput(disp, rootWindow().window(), True);
-#else
-    XRRSelectInput(disp, rootWindow().window(),
-                   RRScreenChangeNotifyMask);
-#endif // X_RRScreenChangeSelectInput
 
+// setup RANDR for this screens root window
+#if defined(HAVE_RANDR1_2)
+    int randr_mask = RRScreenChangeNotifyMask;
+#ifdef RRCrtcChangeNotifyMask
+    randr_mask |= RRCrtcChangeNotifyMask;
+#endif
+#ifdef RROutputChangeNotifyMask
+    randr_mask |= RROutputChangeNotifyMask;
+#endif
+#ifdef RROutputPropertyNotifyMask
+    randr_mask |= RROutputPropertyNotifyMask;
+#endif
+    XRRSelectInput(disp, rootWindow().window(), randr_mask);
+
+#elif defined(HAVE_RANDR)
+    XRRScreenChangeSelectInput(disp, rootWindow().window(), True);
 #endif // HAVE_RANDR
 
+
     _FB_USES_NLS;
 
 #ifdef DEBUG
@@ -406,7 +432,7 @@ BScreen::BScreen(FbTk::ResourceManager &rm,
 
     FbTk::EventManager *evm = FbTk::EventManager::instance();
     evm->add(*this, rootWindow());
-    Keys *keys = Fluxbox::instance()->keys();
+    Keys *keys = fluxbox->keys();
     if (keys)
         keys->registerWindow(rootWindow().window(), *this,
                              Keys::GLOBAL|Keys::ON_DESKTOP);
@@ -467,9 +493,8 @@ BScreen::BScreen(FbTk::ResourceManager &rm,
     // check which desktop we should start on
     unsigned int first_desktop = 0;
     if (m_restart) {
-        Atom net_desktop = XInternAtom(disp, "_NET_CURRENT_DESKTOP", False);
         bool exists;
-        unsigned int ret=static_cast<unsigned int>(rootWindow().cardinalProperty(net_desktop, &exists));
+        unsigned int ret=static_cast<unsigned int>(rootWindow().cardinalProperty(atom_net_desktop, &exists));
         if (exists) {
             if (ret < static_cast<unsigned int>(nr_ws))
                 first_desktop = ret;
@@ -478,10 +503,10 @@ BScreen::BScreen(FbTk::ResourceManager &rm,
 
     changeWorkspaceID(first_desktop);
 
-#ifdef SLIT
+#ifdef USE_SLIT
     m_slit.reset(new Slit(*this, *layerManager().getLayer(ResourceLayer::DESKTOP),
                  fluxbox->getSlitlistFilename().c_str()));
-#endif // SLIT
+#endif // USE_SLIT
 
     rm.unlock();
 
@@ -682,10 +707,10 @@ void BScreen::initWindows() {
     XFree(children);
 
     // now, show slit and toolbar
-#ifdef SLIT
+#ifdef USE_SLIT
     if (slit())
         slit()->show();
-#endif // SLIT
+#endif // USE_SLIT
 
 }
 
@@ -755,29 +780,29 @@ void BScreen::focusedWinFrameThemeReconfigured() {
 }
 
 void BScreen::propertyNotify(Atom atom) {
-    static Atom fbcmd_atom = XInternAtom(FbTk::App::instance()->display(),
-                                         "_FLUXBOX_ACTION", False);
-    if (allowRemoteActions() && atom == fbcmd_atom) {
+
+    if (allowRemoteActions() && atom == atom_fbcmd) {
         Atom xa_ret_type;
         int ret_format;
         unsigned long ret_nitems, ret_bytes_after;
         char *str;
-        if (rootWindow().property(fbcmd_atom, 0l, 64l,
+        if (rootWindow().property(atom_fbcmd, 0l, 64l,
                 True, XA_STRING, &xa_ret_type, &ret_format, &ret_nitems,
                 &ret_bytes_after, (unsigned char **)&str) && str) {
 
             if (ret_bytes_after) {
                 XFree(str);
                 long len = 64 + (ret_bytes_after + 3)/4;
-                rootWindow().property(fbcmd_atom, 0l, len,
+                rootWindow().property(atom_fbcmd, 0l, len,
                     True, XA_STRING, &xa_ret_type, &ret_format, &ret_nitems,
                     &ret_bytes_after, (unsigned char **)&str);
             }
 
             static std::auto_ptr<FbTk::Command<void> > cmd(0);
             cmd.reset(FbTk::CommandParser<void>::instance().parse(str, false));
-            if (cmd.get())
+            if (cmd.get()) {
                 cmd->execute();
+            }
             XFree(str);
 
         }
@@ -843,9 +868,8 @@ void BScreen::cycleFocus(int options, const ClientPattern *pat, bool reverse) {
 }
 
 FbMenu *BScreen::createMenu(const string &label) {
-    FbMenu *menu = new FbMenu(menuTheme(),
-                                  imageControl(),
-                                  *layerManager().getLayer(ResourceLayer::MENU));
+    FbTk::Layer* layer = layerManager().getLayer(ResourceLayer::MENU);
+    FbMenu *menu = new FbMenu(menuTheme(), imageControl(), *layer);
     if (!label.empty())
         menu->setLabel(label);
 
@@ -853,9 +877,8 @@ FbMenu *BScreen::createMenu(const string &label) {
 }
 
 FbMenu *BScreen::createToggleMenu(const string &label) {
-    FbMenu *menu = new ToggleMenu(menuTheme(),
-                                      imageControl(),
-                                      *layerManager().getLayer(ResourceLayer::MENU));
+    FbTk::Layer* layer = layerManager().getLayer(ResourceLayer::MENU);
+    FbMenu *menu = new ToggleMenu(menuTheme(), imageControl(), *layer);
     if (!label.empty())
         menu->setLabel(label);
 
@@ -1168,9 +1191,7 @@ bool BScreen::isKdeDockapp(Window client) const {
     unsigned long *data = 0, uljunk;
     Display *disp = FbTk::App::instance()->display();
     // Check if KDE v2.x dock applet
-    if (XGetWindowProperty(disp, client,
-                           XInternAtom(FbTk::App::instance()->display(),
-                                       "_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR", False),
+    if (XGetWindowProperty(disp, client, atom_kde_systray,
                            0l, 1l, False,
                            XA_WINDOW, &ajunk, &ijunk, &uljunk,
                            &uljunk, (unsigned char **) &data) == Success) {
@@ -1183,11 +1204,9 @@ bool BScreen::isKdeDockapp(Window client) const {
 
     // Check if KDE v1.x dock applet
     if (!iskdedockapp) {
-        Atom kwm1 = XInternAtom(FbTk::App::instance()->display(),
-                                "KWM_DOCKWINDOW", False);
         if (XGetWindowProperty(disp, client,
-                               kwm1, 0l, 1l, False,
-                               kwm1, &ajunk, &ijunk, &uljunk,
+                               atom_kwm1, 0l, 1l, False,
+                               atom_kwm1, &ajunk, &ijunk, &uljunk,
                                &uljunk, (unsigned char **) &data) == Success && data) {
             iskdedockapp = (data && data[0] != 0);
             XFree((void *) data);
@@ -1205,15 +1224,15 @@ bool BScreen::addKdeDockapp(Window client) {
     FbTk::EventManager *evm = FbTk::EventManager::instance();
 
     AtomHandler* handler = 0;
-#if USE_TOOLBAR
+#if USE_SYSTRAY
     handler = Fluxbox::instance()->getAtomHandler(SystemTray::getNetSystemTrayAtom(screenNumber()));
 #endif
     if (handler == 0) {
-#ifdef SLIT
+#ifdef USE_SLIT
         if (slit() != 0 && slit()->acceptKdeDockapp())
             slit()->addClient(client);
         else
-#endif // SLIT
+#endif // USE_SLIT
             return false;
     } else {
         // this handler is a special case
@@ -1242,10 +1261,10 @@ FluxboxWindow *BScreen::createWindow(Window client) {
 
     if (winclient->initial_state == WithdrawnState) {
         delete winclient;
-#ifdef SLIT
+#ifdef USE_SLIT
         if (slit() && !isKdeDockapp(client))
             slit()->addClient(client);
-#endif // SLIT
+#endif // USE_SLIT
         return 0;
     }
 
@@ -1485,6 +1504,55 @@ void BScreen::addManagedResource(FbTk::Resource_base *resource) {
     m_managed_resources.push_back(resource);
 }
 
+int BScreen::getGap(int head, const char type) {
+    return type == 'w' ? getXGap(head) : getYGap(head);
+}
+
+int BScreen::calRelativeSize(int head, int i, char type) {
+    // return floor(i * getGap(head, type) / 100 + 0.5);
+    return FbTk::RelCalcHelper::calPercentageValueOf(i, getGap(head, type));
+}
+int BScreen::calRelativeWidth(int head, int i) {
+    return calRelativeSize(head, i, 'w');
+}
+int BScreen::calRelativeHeight(int head, int i) {
+    return calRelativeSize(head, i, 'h');
+}
+
+int BScreen::calRelativePosition(int head, int i, char type) {
+    int max = type == 'w' ? maxLeft(head) : maxTop(head);
+    // return floor((i - min) / getGap(head, type)  * 100 + 0.5);
+    return FbTk::RelCalcHelper::calPercentageOf((i - max), getGap(head, type));
+}
+// returns a pixel, which is relative to the width of the screen
+// screen starts from 0, 1000 px width, if i is 10 then it should return 100
+int BScreen::calRelativePositionWidth(int head, int i) {
+    return calRelativePosition(head, i, 'w');
+}
+// returns a pixel, which is relative to the height of th escreen
+// screen starts from 0, 1000 px height, if i is 10 then it should return 100
+int BScreen::calRelativePositionHeight(int head, int i) {
+    return calRelativePosition(head, i, 'h');
+}
+
+int BScreen::calRelativeDimension(int head, int i, char type) {
+    // return floor(i / getGap(head, type) * 100 + 0.5);
+    return FbTk::RelCalcHelper::calPercentageOf(i, getGap(head, type));
+  }
+int BScreen::calRelativeDimensionWidth(int head, int i) {
+    return calRelativeDimension(head, i, 'w');
+}
+int BScreen::calRelativeDimensionHeight(int head, int i) {
+    return calRelativeDimension(head, i, 'h');
+}
+
+float BScreen::getXGap(int head) {
+    return maxRight(head) - maxLeft(head);
+}
+float BScreen::getYGap(int head) {
+    return maxBottom(head) - maxTop(head);
+}
+
 void BScreen::setupConfigmenu(FbTk::Menu &menu) {
     _FB_USES_NLS;
 
@@ -1548,9 +1616,20 @@ void BScreen::setupConfigmenu(FbTk::Menu &menu) {
             "Focus New Windows", "Focus newly created windows"),
             m_resource_manager.getResource<bool>(name() + ".focusNewWindows"),
             saverc_cmd));
+    } catch (FbTk::ResourceException & e) {
+        cerr<<e.what()<<endl;
+    }
+   
+#ifdef XINERAMA
+    try {
+        focus_menu->insert(new FbTk::BoolMenuItem(_FB_XTEXT(Configmenu, FocusSameHead,
+            "Keep Head", "Only revert focus on same head"),
+            m_resource_manager.getResource<bool>(name() + ".focusSameHead"),
+            saverc_cmd));
     } catch (FbTk::ResourceException e) {
         cerr<<e.what()<<endl;
     }
+#endif // XINERAMA
 
     _BOOLITEM(*focus_menu, Configmenu, AutoRaise,
               "Auto Raise", "Auto Raise windows on sloppy",
@@ -1981,9 +2060,9 @@ void BScreen::clearHeads() {
     if (!hasXinerama()) return;
 
     for (Workspaces::iterator i = m_workspaces_list.begin();
-            i != m_workspaces_list.end(); i++) {
+            i != m_workspaces_list.end(); ++i) {
         for (Workspace::Windows::iterator win = (*i)->windowList().begin();
-                win != (*i)->windowList().end(); win++) {
+                win != (*i)->windowList().end(); ++win) {
 
             FluxboxWindow& w = *(*win);
 
