@@ -170,14 +170,12 @@ Toolbar::Frame::Frame(FbTk::EventHandler &evh, int screen_num):
 {
 
     FbTk::EventManager &evm = *FbTk::EventManager::instance();
-    // add windows to eventmanager
     evm.add(evh, window);
 
 }
 
 Toolbar::Frame::~Frame() {
     FbTk::EventManager &evm = *FbTk::EventManager::instance();
-    // remove windows from eventmanager
     evm.remove(window);
 }
 
@@ -227,6 +225,8 @@ Toolbar::Toolbar(BScreen &scrn, FbTk::Layer &layer, size_t width):
     m_resize_lock(false) {
     _FB_USES_NLS;
 
+    frame.window.setWindowRole("fluxbox-toolbar");
+
     // get this on antialias change
     m_signal_tracker.join(screen().reconfigureSig(),
             FbTk::MemFunIgnoreArgs(*this, &Toolbar::reconfigure));
@@ -258,7 +258,7 @@ Toolbar::Toolbar(BScreen &scrn, FbTk::Layer &layer, size_t width):
     frame.grab_x = frame.grab_y = 0;
 
     // setup hide timer
-    m_hide_timer.setTimeout(Fluxbox::instance()->getAutoRaiseDelay());
+    m_hide_timer.setTimeout(Fluxbox::instance()->getAutoRaiseDelay() * FbTk::FbTime::IN_MILLISECONDS);
     FbTk::RefCount<FbTk::Command<void> > toggle_hidden(new FbTk::SimpleCommand<Toolbar>(*this, &Toolbar::toggleHidden));
     m_hide_timer.setCommand(toggle_hidden);
     m_hide_timer.fireOnce(true);
